@@ -13,24 +13,26 @@
 # limitations under the License.
 
 import six
+import toolz
 
 from ibis.expr.datatypes import Schema  # noqa
 from ibis.expr.types import (Expr,  # noqa
-                             ValueExpr, ScalarExpr, ArrayExpr,
+                             ValueExpr, ScalarExpr, ColumnExpr,
                              TableExpr,
-                             NumericValue, NumericArray,
+                             NumericValue, NumericColumn,
                              IntegerValue,
-                             Int8Value, Int8Scalar, Int8Array,
-                             Int16Value, Int16Scalar, Int16Array,
-                             Int32Value, Int32Scalar, Int32Array,
-                             Int64Value, Int64Scalar, Int64Array,
+                             Int8Value, Int8Scalar, Int8Column,
+                             Int16Value, Int16Scalar, Int16Column,
+                             Int32Value, Int32Scalar, Int32Column,
+                             Int64Value, Int64Scalar, Int64Column,
                              NullScalar,
-                             BooleanValue, BooleanScalar, BooleanArray,
-                             FloatValue, FloatScalar, FloatArray,
-                             DoubleValue, DoubleScalar, DoubleArray,
-                             StringValue, StringScalar, StringArray,
-                             DecimalValue, DecimalScalar, DecimalArray,
-                             TimestampValue, TimestampScalar, TimestampArray,
+                             BooleanValue, BooleanScalar, BooleanColumn,
+                             FloatValue, FloatScalar, FloatColumn,
+                             DoubleValue, DoubleScalar, DoubleColumn,
+                             StringValue, StringScalar, StringColumn,
+                             DecimalValue, DecimalScalar, DecimalColumn,
+                             TimestampValue, TimestampScalar, TimestampColumn,
+                             ArrayValue, ArrayScalar, ArrayColumn,
                              CategoryValue, unnamed, as_value_expr, literal,
                              null, sequence)
 
@@ -278,7 +280,7 @@ def count(expr, where=None):
     counts : int64 type
     """
     op = expr.op()
-    if isinstance(op, _ops.DistinctArray):
+    if isinstance(op, _ops.DistinctColumn):
         if where is not None:
             raise NotImplementedError
         result = op.count().to_expr()
@@ -345,6 +347,13 @@ def _boolean_binary_op(name, klass):
     return f
 
 
+def _boolean_unary_op(name, klass):
+    def f(self):
+        return klass(self).to_expr()
+    f.__name__ = name
+    return f
+
+
 def _boolean_binary_rop(name, klass):
     def f(self, other):
         other = as_value_expr(other)
@@ -401,6 +410,7 @@ def cast(arg, target_type):
             pass
         return result
 
+
 cast.__doc__ = """
 Cast value(s) to indicated data type. Values that cannot be
 successfully casted
@@ -797,7 +807,9 @@ _generic_value_methods = dict(
     __ge__=_binop_expr('__ge__', _ops.GreaterEqual),
     __gt__=_binop_expr('__gt__', _ops.Greater),
     __le__=_binop_expr('__le__', _ops.LessEqual),
-    __lt__=_binop_expr('__lt__', _ops.Less)
+    __lt__=_binop_expr('__lt__', _ops.Less),
+    collect=_unary_op('collect', _ops.ArrayCollect),
+    identical_to=_binop_expr('identical_to', _ops.IdenticalTo),
 )
 
 
@@ -819,10 +831,15 @@ first = _unary_op('first', _ops.FirstValue)
 last = _unary_op('last', _ops.LastValue)
 rank = _unary_op('rank', _ops.MinRank)
 dense_rank = _unary_op('dense_rank', _ops.DenseRank)
+percent_rank = _unary_op('percent_rank', _ops.PercentRank)
 cummin = _unary_op('cummin', _ops.CumulativeMin)
 cummax = _unary_op('cummax', _ops.CumulativeMax)
 
 
+def ntile(arg, buckets):
+    return _ops.NTile(arg, buckets).to_expr()
+
+
 def nth(arg, k):
     """
     Analytic operation computing nth value from start of sequence
@@ -846,7 +863,7 @@ def distinct(arg):
     in conjunction with other array expressions from the same context
     (because it's a cardinality-modifying pseudo-reduction).
     """
-    op = _ops.DistinctArray(arg)
+    op = _ops.DistinctColumn(arg)
     return op.to_expr()
 
 
@@ -950,7 +967,7 @@ def expr_list(exprs):
     return ir.ExpressionList(exprs).to_expr()
 
 
-_generic_array_methods = dict(
+_generic_column_methods = dict(
     bottomk=bottomk,
     distinct=distinct,
     nunique=nunique,
@@ -968,7 +985,9 @@ _generic_array_methods = dict(
     last=last,
     dense_rank=dense_rank,
     rank=rank,
+    percent_rank=percent_rank,
     # nth=nth,
+    ntile=ntile,
     lag=lag,
     lead=lead,
     cummin=cummin,
@@ -977,7 +996,7 @@ _generic_array_methods = dict(
 
 
 _add_methods(ValueExpr, _generic_value_methods)
-_add_methods(ArrayExpr, _generic_array_methods)
+_add_methods(ColumnExpr, _generic_column_methods)
 
 
 # ---------------------------------------------------------------------
@@ -1128,7 +1147,7 @@ def variance(arg, where=None, how='sample'):
     return expr
 
 
-_numeric_array_methods = dict(
+_numeric_column_methods = dict(
     mean=mean,
     cummean=cummean,
 
@@ -1146,7 +1165,7 @@ _numeric_array_methods = dict(
 _add_methods(NumericValue, _numeric_value_methods)
 _add_methods(IntegerValue, _integer_value_methods)
 
-_add_methods(NumericArray, _numeric_array_methods)
+_add_methods(NumericColumn, _numeric_column_methods)
 
 
 # ----------------------------------------------------------------------
@@ -1177,11 +1196,12 @@ _boolean_value_methods = dict(
     __xor__=_boolean_binary_op('__xor__', _ops.Xor),
     __rand__=_boolean_binary_rop('__rand__', _ops.And),
     __ror__=_boolean_binary_rop('__ror__', _ops.Or),
-    __rxor__=_boolean_binary_rop('__rxor__', _ops.Xor)
+    __rxor__=_boolean_binary_rop('__rxor__', _ops.Xor),
+    __invert__=_boolean_unary_op('__invert__', _ops.Not),
 )
 
 
-_boolean_array_methods = dict(
+_boolean_column_methods = dict(
     any=_unary_op('any', _ops.Any),
     notany=_unary_op('notany', _ops.NotAny),
     all=_unary_op('all', _ops.All),
@@ -1192,7 +1212,7 @@ _boolean_array_methods = dict(
 
 
 _add_methods(BooleanValue, _boolean_value_methods)
-_add_methods(BooleanArray, _boolean_array_methods)
+_add_methods(BooleanColumn, _boolean_column_methods)
 
 
 # ---------------------------------------------------------------------
@@ -1588,6 +1608,45 @@ _string_value_methods = dict(
 _add_methods(StringValue, _string_value_methods)
 
 
+# ---------------------------------------------------------------------
+# Array API
+
+
+def _array_slice(array, index):
+    if isinstance(index, slice):
+        start = index.start
+        stop = index.stop
+        if ((start is not None and start < 0) or
+                (stop is not None and stop < 0)):
+            raise ValueError('negative slicing not yet supported')
+
+        step = index.step
+
+        if step is not None and step != 1:
+            raise NotImplementedError('step can only be 1')
+
+        result = _ops.ArraySlice(
+            array,
+            start + 1 if start is not None else 1,
+            stop if stop is not None else array.length(),
+        )
+    else:
+        result = _ops.ArrayIndex(array, index + 1)
+    return result.to_expr()
+
+
+_array_column_methods = dict(
+    length=_unary_op('length', _ops.ArrayLength),
+    __getitem__=_array_slice,
+    __add__=_binop_expr('__add__', _ops.ArrayConcat),
+    __radd__=toolz.flip(_binop_expr('__radd__', _ops.ArrayConcat)),
+    __mul__=_binop_expr('__mul__', _ops.ArrayRepeat),
+    __rmul__=_binop_expr('__rmul__', _ops.ArrayRepeat),
+)
+
+_add_methods(ArrayValue, _array_column_methods)
+
+
 # ---------------------------------------------------------------------
 # Timestamp API
 
@@ -1895,12 +1954,12 @@ def _table_distinct(self):
 def _table_limit(table, n, offset=0):
     """
     Select the first n rows at beginning of table (may not be deterministic
-    depending on implementatino and presence of a sorting).
+    depending on implementation and presence of a sorting).
 
     Parameters
     ----------
     n : int
-      Rows to include
+      Number of rows to include
     offset : int, default 0
       Number of rows to skip first
 
@@ -1912,6 +1971,27 @@ def _table_limit(table, n, offset=0):
     return TableExpr(op)
 
 
+def _head(table, n=5):
+    """
+    Select the first n rows at beginning of a table (may not be deterministic
+    depending on implementation and presence of a sorting).
+
+    Parameters
+    ----------
+    n : int
+      Number of rows to include, defaults to 5
+
+    Returns
+    -------
+    limited : TableExpr
+
+    See Also
+    --------
+    ibis.expr.types.TableExpr.limit
+    """
+    return _table_limit(table, n=n)
+
+
 def _table_sort_by(table, sort_exprs):
     """
     Sort table by the indicated column expressions and sort orders
@@ -2134,6 +2214,7 @@ _table_methods = dict(
     drop=_table_drop,
     info=_table_info,
     limit=_table_limit,
+    head=_head,
     set_column=_table_set_column,
     filter=filter,
     materialize=_table_materialize,
