@@ -188,10 +188,10 @@ class ExprFormatter(object):
             self.memo.observe(parent, formatter=self._format_node)
 
         table_formatted = self.memo.get_alias(parent)
-        table_formatted = '\n' + self._indent(table_formatted)
+        table_formatted = self._indent(table_formatted)
 
         type_display = self._get_type_display(self.expr)
-        return ("Column[{0}] '{1}' from table {2}"
+        return ("Column[{0}] '{1}' from table\n{2}"
                 .format(type_display, col.name, table_formatted))
 
     def _format_node(self, expr):
@@ -209,9 +209,9 @@ class ExprFormatter(object):
 
             formatted_args.append(result)
 
-        arg_names = getattr(op, '_arg_names', None)
+        arg_names = op._arg_names
 
-        if arg_names is None:
+        if not arg_names:
             for arg in op.args:
                 if isinstance(arg, list):
                     for x in arg:
@@ -241,7 +241,6 @@ class ExprFormatter(object):
         opname = type(op).__name__
         type_display = self._get_type_display(expr)
         opline = '%s[%s]' % (opname, type_display)
-
         return '\n'.join([opline] + formatted_args)
 
     def _format_subexpr(self, expr):
@@ -256,17 +255,4 @@ class ExprFormatter(object):
         if expr is None:
             expr = self.expr
 
-        if isinstance(expr, ir.TableExpr):
-            return 'table'
-        elif isinstance(expr, ir.ArrayExpr):
-            return 'array(%s)' % expr.type()
-        elif isinstance(expr, ir.SortExpr):
-            return 'array-sort'
-        elif isinstance(expr, (ir.ScalarExpr, ir.AnalyticExpr)):
-            return '%s' % expr.type()
-        elif isinstance(expr, ir.ExprList):
-            list_args = [self._get_type_display(arg)
-                         for arg in expr.op().args]
-            return ', '.join(list_args)
-        else:
-            raise NotImplementedError
+        return expr._type_display()
