@@ -21,8 +21,8 @@ from ibis.compat import py_string
 from ibis.expr.datatypes import HasSchema, Schema
 from ibis.expr.rules import value, string, number, integer, boolean, list_of
 from ibis.expr.types import (Node, as_value_expr, Expr,
-                             ValueExpr, ArrayExpr, TableExpr,
-                             ValueNode, _safe_repr)
+                             ValueExpr, ColumnExpr, TableExpr,
+                             ValueOp, _safe_repr)
 import ibis.common as com
 import ibis.expr.datatypes as dt
 import ibis.expr.rules as rules
@@ -30,13 +30,6 @@ import ibis.expr.types as ir
 import ibis.util as util
 
 
-def _arg_getter(i):
-    @property
-    def arg_accessor(self):
-        return self.args[i]
-    return arg_accessor
-
-
 class TableNode(Node):
 
     def get_type(self, name):
@@ -70,30 +63,6 @@ def find_all_base_tables(expr, memo=None):
     return memo
 
 
-class ValueOperationMeta(type):
-
-    def __new__(cls, name, parents, dct):
-
-        if 'input_type' in dct:
-            sig = dct['input_type']
-            if not isinstance(sig, rules.TypeSignature):
-                dct['input_type'] = sig = rules.signature(sig)
-
-                for i, t in enumerate(sig.types):
-                    if t.name is None:
-                        continue
-
-                    if t.name not in dct:
-                        dct[t.name] = _arg_getter(i)
-
-        return super(ValueOperationMeta, cls).__new__(cls, name, parents, dct)
-
-
-class ValueOp(six.with_metaclass(ValueOperationMeta, ValueNode)):
-
-    pass
-
-
 class PhysicalTable(TableNode, HasSchema):
 
     def blocks(self):
@@ -138,7 +107,7 @@ class SQLQueryResult(TableNode, HasSchema):
         return True
 
 
-class TableArrayView(ValueNode):
+class TableArrayView(ValueOp):
 
     """
     (Temporary?) Helper operation class for SQL translation (fully formed table
@@ -857,7 +826,7 @@ class Lead(ShiftBase):
 class RankBase(AnalyticOp):
 
     def output_type(self):
-        return ir.Int64Array
+        return ir.Int64Column
 
 
 class MinRank(RankBase):
@@ -878,7 +847,7 @@ class MinRank(RankBase):
 
     Returns
     -------
-    ranks : Int64Array, starting from 0
+    ranks : Int64Column, starting from 0
     """
 
     # Equivalent to SQL RANK()
@@ -903,7 +872,7 @@ class DenseRank(RankBase):
 
     Returns
     -------
-    ranks : Int64Array, starting from 0
+    ranks : Int64Column, starting from 0
     """
 
     # Equivalent to SQL DENSE_RANK()
@@ -930,7 +899,7 @@ class RowNumber(RankBase):
 
     Returns
     -------
-    row_number : Int64Array, starting from 0
+    row_number : Int64Column, starting from 0
     """
 
     # Equivalent to SQL ROW_NUMBER()
@@ -979,11 +948,15 @@ class CumulativeMin(CumulativeOp):
 
 
 class PercentRank(AnalyticOp):
-    pass
+
+    input_type = [rules.array]
+    output_type = rules.shape_like_arg(0, 'double')
 
 
 class NTile(AnalyticOp):
-    pass
+
+    input_type = [rules.array, rules.integer(name='buckets')]
+    output_type = rules.shape_like_arg(0, 'int64')
 
 
 class FirstValue(AnalyticOp):
@@ -1044,7 +1017,7 @@ class Distinct(TableNode, HasSchema):
         return True
 
 
-class DistinctArray(ValueNode):
+class DistinctColumn(ValueOp):
 
     """
     COUNT(DISTINCT ...) is really just syntactic suger, but we provide a
@@ -1057,7 +1030,7 @@ class DistinctArray(ValueNode):
 
     def __init__(self, arg):
         self.arg = arg
-        ValueNode.__init__(self, arg)
+        ValueOp.__init__(self, arg)
 
     def output_type(self):
         return type(self.arg)
@@ -1093,7 +1066,7 @@ class Any(ValueOp):
         return len(roots) < 2
 
     def output_type(self):
-        return ir.BooleanScalar if self._reduction else ir.BooleanArray
+        return ir.BooleanScalar if self._reduction else ir.BooleanColumn
 
     def negate(self):
         return NotAny(self.args[0])
@@ -1400,7 +1373,7 @@ class Join(TableNode):
             elif not isinstance(pred, ir.Expr):
                 raise NotImplementedError
 
-            if not isinstance(pred, ir.BooleanArray):
+            if not isinstance(pred, ir.BooleanColumn):
                 raise com.ExpressionError('Join predicate must be comparison')
 
             preds = L.unwrap_ands(pred)
@@ -1734,18 +1707,7 @@ class Selection(TableNode, HasSchema):
         if self.equals(other):
             return True
 
-        table = self.table
-        exist_layers = False
-        op = table.op()
-        while not (op.blocks() or isinstance(op, Join)):
-            table = table.op().table
-            exist_layers = True
-
-        if exist_layers:
-            reboxed = Selection(table, self.selections)
-            return reboxed.is_ancestor(other)
-        else:
-            return False
+        return False
 
     # Operator combination / fusion logic
 
@@ -1792,7 +1754,8 @@ class AggregateSelection(object):
     def _attempt_pushdown(self):
         metrics_valid, lowered_metrics = self._pushdown_exprs(self.metrics)
         by_valid, lowered_by = self._pushdown_exprs(self.by)
-        having_valid, lowered_having = self._pushdown_exprs(self.having or None)
+        having_valid, lowered_having = self._pushdown_exprs(
+            self.having or None)
 
         if metrics_valid and by_valid and having_valid:
             return Aggregation(self.op.table, lowered_metrics,
@@ -2004,6 +1967,12 @@ class LogicalBinaryOp(BinaryOp):
         return rules.shape_like_args(self.args, 'boolean')
 
 
+class Not(UnaryOp):
+
+    input_type = [rules.boolean]
+    output_type = rules.shape_like_arg(0, 'boolean')
+
+
 class Modulus(BinaryOp):
 
     def output_type(self):
@@ -2068,6 +2037,10 @@ class Less(Comparison):
     pass
 
 
+class IdenticalTo(Comparison):
+    pass
+
+
 class Between(BooleanValueOp):
 
     input_type = [rules.value, rules.value(name='lower_bound'),
@@ -2079,7 +2052,7 @@ class Between(BooleanValueOp):
 
     def _assert_can_compare(self):
         expr, lower, upper = self.args
-        if (not expr._can_compare(lower) or not expr._can_compare(upper)):
+        if not expr._can_compare(lower) or not expr._can_compare(upper):
             raise TypeError('Arguments are not comparable')
 
 
@@ -2088,7 +2061,7 @@ class Contains(BooleanValueOp):
     def __init__(self, value, options):
         self.value = as_value_expr(value)
         self.options = as_value_expr(options)
-        BooleanValueOp.__init__(self, self.value, self.options)
+        super(Contains, self).__init__(self.value, self.options)
 
     def output_type(self):
         all_args = [self.value]
@@ -2096,7 +2069,7 @@ class Contains(BooleanValueOp):
         options = self.options.op()
         if isinstance(options, ir.ValueList):
             all_args += options.values
-        elif isinstance(self.options, ArrayExpr):
+        elif isinstance(self.options, ColumnExpr):
             all_args += [self.options]
         else:
             raise TypeError(type(options))
@@ -2108,7 +2081,7 @@ class NotContains(Contains):
     pass
 
 
-class ReplaceValues(ValueNode):
+class ReplaceValues(ValueOp):
 
     """
     Apply a multi-value replacement on a particular column. As an example from
@@ -2162,16 +2135,13 @@ class TopKExpr(ir.AnalyticExpr):
         return agg.sort_by([(by.get_name(), False)]).limit(op.k)
 
 
-class SummaryFilter(ValueNode):
-
-    def __init__(self, expr):
-        ValueNode.__init__(self, expr)
+class SummaryFilter(ValueOp):
 
     def output_type(self):
-        return ir.BooleanArray
+        return ir.BooleanColumn
 
 
-class TopK(ValueNode):
+class TopK(ValueOp):
 
     def blocks(self):
         return True
@@ -2180,7 +2150,7 @@ class TopK(ValueNode):
         if by is None:
             by = arg.count()
 
-        if not isinstance(arg, ArrayExpr):
+        if not isinstance(arg, ColumnExpr):
             raise TypeError(arg)
 
         if not isinstance(k, int) or k < 0:
@@ -2197,9 +2167,7 @@ class TopK(ValueNode):
 
 
 class Constant(ValueOp):
-
-    def __init__(self):
-        ValueOp.__init__(self, [])
+    pass
 
 
 class TimestampNow(Constant):
@@ -2353,3 +2321,64 @@ class TimestampDelta(ValueOp):
 
     input_type = [rules.timestamp, rules.timedelta(name='offset')]
     output_type = rules.shape_like_arg(0, 'timestamp')
+
+
+class ArrayLength(ValueOp):
+
+    input_type = [rules.array_column(dt.any)]
+    output_type = rules.shape_like_arg(0, 'int64')
+
+
+class ArraySlice(ValueOp):
+
+    input_type = [
+        rules.array_column(dt.any),
+        rules.integer(name='start'),
+        rules.integer(name='stop')
+    ]
+    output_type = rules.array_output(lambda self: self.args[0].type())
+
+
+class ArrayIndex(ValueOp):
+
+    input_type = [rules.array_column(dt.any), rules.integer(name='index')]
+    output_type = rules.array_output(
+        lambda self: self.args[0].type().value_type
+    )
+
+
+def _array_binop_invariant_output_type(self):
+    """Check whether two arrays in an array OP array binary operation have
+    the same type.
+    """
+    args = self.args
+    left_type = args[0].type()
+    right_type = args[1].type()
+    if left_type != right_type:
+        raise TypeError(
+            'Array types must match exactly in a {} operation. '
+            'Left type {} != Right type {}'.format(
+                type(self).__name__, left_type, right_type
+            )
+        )
+    return left_type
+
+
+class ArrayConcat(ValueOp):
+
+    input_type = [rules.array_column(dt.any), rules.array_column(dt.any)]
+    output_type = rules.array_output(_array_binop_invariant_output_type)
+
+
+class ArrayRepeat(ValueOp):
+
+    input_type = [rules.array_column(dt.any), integer(name='times')]
+    output_type = rules.array_output(lambda self: self.args[0].type())
+
+
+class ArrayCollect(Reduction):
+
+    input_type = [rules.array]
+    output_type = rules.scalar_output(
+        lambda self: dt.Array(self.args[0].type())
+    )
