@@ -235,7 +235,7 @@ def shape_like(arg, out_type):
 
 def shape_like_args(args, out_type):
     out_type = dt.validate_type(out_type)
-    if util.any_of(args, ir.ArrayExpr):
+    if util.any_of(args, ir.ColumnExpr):
         return out_type.array_type()
     else:
         return out_type.scalar_type()
@@ -246,7 +246,7 @@ def is_table(e):
 
 
 def is_array(e):
-    return isinstance(e, ir.ArrayExpr)
+    return isinstance(e, ir.ColumnExpr)
 
 
 def is_scalar(e):
@@ -254,7 +254,7 @@ def is_scalar(e):
 
 
 def is_collection(expr):
-    return isinstance(expr, (ir.ArrayExpr, ir.TableExpr))
+    return isinstance(expr, (ir.ColumnExpr, ir.TableExpr))
 
 
 class Argument(object):
@@ -549,7 +549,7 @@ def value_typed_as(types, **arg_kwds):
 
 
 def array(value_type=None, name=None, optional=False):
-    array_checker = ValueTyped(ir.ArrayExpr, 'not an array expr',
+    array_checker = ValueTyped(ir.ColumnExpr, 'not an array expr',
                                name=name,
                                optional=optional)
     if value_type is None:
@@ -566,7 +566,7 @@ def scalar(name=None, optional=False):
 
 
 def collection(name=None, optional=False):
-    return ValueTyped((ir.ArrayExpr, ir.TableExpr), 'not a collection',
+    return ValueTyped((ir.ColumnExpr, ir.TableExpr), 'not a collection',
                       name=name, optional=optional)
 
 
@@ -622,6 +622,14 @@ def string(**arg_kwds):
     return ValueTyped(dt.string, 'not string', **arg_kwds)
 
 
+def array_column(value_type):
+    return lambda **arg_kwds: ValueTyped(
+        dt.Array(value_type),
+        'not array with value_type {0}'.format(value_type),
+        **arg_kwds
+    )
+
+
 def boolean(**arg_kwds):
     return ValueTyped(dt.boolean, 'not string', **arg_kwds)
 
@@ -630,23 +638,79 @@ def one_of(args, **arg_kwds):
     return OneOf(args, **arg_kwds)
 
 
+def instance_of(type_, **arg_kwds):
+    fail_message = 'not a {0}'.format(str(type_))
+    return AnyTyped(type_, fail_message, **arg_kwds)
+
+
 class StringOptions(Argument):
 
-    def __init__(self, options, **arg_kwds):
-        self.options = options
-        Argument.__init__(self, **arg_kwds)
+    def __init__(self, options, case_sensitive=True, **arg_kwds):
+        if not case_sensitive:
+            (is_lower, _), = Counter(map(str.islower, options)).most_common(1)
+            self._preferred_case = str.lower if is_lower else str.upper
+            self.options = list(map(self._preferred_case, options))
+        else:
+            self.options = options
+        self.case_sensitive = case_sensitive
+        super(StringOptions, self).__init__(**arg_kwds)
 
     def _validate(self, args, i):
         arg = args[i]
+
+        if not self.case_sensitive:
+            arg = self._preferred_case(arg)
+
         if arg not in self.options:
-            raise IbisTypeError('{0} not among options {1}'
-                                .format(arg, repr(self.options)))
+            raise IbisTypeError(
+                '{} not among options {}'.format(arg, self.options)
+            )
         return arg
 
 
 string_options = StringOptions
 
 
+class Enum(Argument):
+    def __init__(self, enum, **arg_kwds):
+        super(Enum, self).__init__(**arg_kwds)
+        self.enum = enum
+
+    def _validate(self, args, i):
+        arg = args[i]
+
+        # if our passed value wasn't specified directly from the enum
+        if not isinstance(arg, self.enum):
+            value_set = {}
+            for key, value in self.enum.__members__.items():
+                value_set.setdefault(value.value, []).append(key)
+
+            # not in the value_set, so can't be valid
+            if arg not in value_set:
+                raise IbisTypeError(
+                    ('Value {} is not a member of the {} enum, '
+                     'whose values are {}').format(
+                         arg, self.enum.__name__, list(self.enum)
+                    )
+                )
+
+            # if it's in the value set and the value set has duplicates, then
+            # we can't validate it because we don't know which one the user
+            # meant
+            if len(value_set[arg]) > 1:
+                raise IbisTypeError(
+                    (
+                        'Value {0} is a member of {1}, but {1} is not unique. '
+                        'Please explicitly pass the desired enum attribute.'
+                    ).format(arg, self.enum.__name__)
+                )
+            return self.enum[value_set[arg].pop()]
+        return arg
+
+
+enum = Enum
+
+
 class ListOf(Argument):
 
     def __init__(self, value_type, min_length=0, **arg_kwds):
