@@ -38,10 +38,25 @@ class TestExprFormatting(unittest.TestCase):
         self.table = ibis.table(self.schema)
         self.con = MockConnection()
 
+    def test_format_custom_expr(self):
+        from ibis.expr.types import Expr, Literal
+
+        class CustomExpr(Expr):
+            def _type_display(self):
+                return 'my-custom'
+
+        op = Literal(5)
+        expr = CustomExpr(op)
+
+        result = repr(expr)
+        expected = """Literal[my-custom]
+  5"""
+        assert result == expected
+
     def test_format_table_column(self):
         # GH #507
         result = repr(self.table.f)
-        assert 'Column[array(double)]' in result
+        assert 'Column[double*]' in result
 
     def test_format_projection(self):
         # This should produce a ref to the projection
@@ -202,3 +217,23 @@ class TestExprFormatting(unittest.TestCase):
 
         # Join, and one for each aggregation
         assert result.count('predicates') == 3
+
+
+def test_argument_repr_shows_name():
+    t = ibis.table([('a', 'int64')])
+    expr = t.a.nullif(2)
+    result = repr(expr)
+    expected = """\
+ref_0
+UnboundTable[table]
+  name: None
+  schema:
+    a : int64
+
+NullIf[int64*]
+  a = Column[int64*] 'a' from table
+    ref_0
+  null_if_expr:
+    Literal[int8]
+      2"""
+    assert result == expected
