@@ -14,7 +14,7 @@
 
 import pytest
 
-from ibis.expr.types import ArrayExpr, TableExpr, RelationError
+from ibis.expr.types import ColumnExpr, TableExpr, RelationError
 from ibis.common import ExpressionError
 from ibis.expr.datatypes import array_type
 import ibis.expr.api as api
@@ -22,9 +22,6 @@ import ibis.expr.types as ir
 import ibis.expr.operations as ops
 import ibis
 
-from ibis.compat import unittest
-from ibis.expr.tests.mocks import MockConnection
-
 import ibis.common as com
 import ibis.config as config
 
@@ -68,7 +65,7 @@ def test_getitem_column_select(table, schema_dict):
         col = table[k]
 
         # Make sure it's the right type
-        assert isinstance(col, ArrayExpr)
+        assert isinstance(col, ColumnExpr)
         assert isinstance(col, array_type(v))
 
         # Ensure we have a field selection with back-reference to the table
@@ -86,7 +83,7 @@ def test_getitem_attribute(table):
     # Project and add a name that conflicts with a TableExpr built-in
     # attribute
     view = table[[table, table['a'].name('schema')]]
-    assert not isinstance(view.schema, ArrayExpr)
+    assert not isinstance(view.schema, ColumnExpr)
 
 
 def test_projection(table):
@@ -156,11 +153,12 @@ def test_projection_unnamed_literal_interactive_blowup(con):
             assert 'named' in e.args[0]
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_projection_of_aggregated(table):
     # Fully-formed aggregations "block"; in a projection, column
     # expressions referencing table expressions below the aggregation are
     # invalid.
-    pass
+    assert False
 
 
 def test_projection_with_star_expr(table):
@@ -230,11 +228,13 @@ def test_add_column(table):
     assert_equal(result, expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_add_column_scalar_expr(table):
     # Check literals, at least
-    pass
+    assert False
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_add_column_aggregate_crossjoin(table):
     # A new column that depends on a scalar value produced by this or some
     # other table.
@@ -245,7 +245,7 @@ def test_add_column_aggregate_crossjoin(table):
     #
     # Here, VAL could be something produced by aggregating table1 or any
     # other table for that matter.
-    pass
+    assert False
 
 
 def test_add_column_existing_projection(table):
@@ -461,7 +461,7 @@ def test_sum_expr_basics(table, int_col):
     # Impala gives bigint for all integer types
     ex_class = api.Int64Scalar
     result = table[int_col].sum()
-    assert isinstance(result, api.Int64Scalar)
+    assert isinstance(result, ex_class)
     assert isinstance(result.op(), ops.Sum)
 
 
@@ -469,7 +469,7 @@ def test_sum_expr_basics_floats(table, float_col):
     # Impala gives double for all floating point types
     ex_class = api.DoubleScalar
     result = table[float_col].sum()
-    assert isinstance(result, api.DoubleScalar)
+    assert isinstance(result, ex_class)
     assert isinstance(result.op(), ops.Sum)
 
 
@@ -544,9 +544,10 @@ def test_summary_expand_list(table):
     assert_equal(result, expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_aggregate_invalid(table):
     # Pass a non-aggregation or non-scalar expr
-    pass
+    assert False
 
 
 def test_filter_aggregate_pushdown_predicate(table):
@@ -567,8 +568,9 @@ def test_filter_aggregate_pushdown_predicate(table):
     assert_equal(filtered, expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_filter_aggregate_partial_pushdown(table):
-    pass
+    assert False
 
 
 def test_aggregate_post_predicate(table):
@@ -608,8 +610,9 @@ def test_group_by_kwargs(table):
     assert_equal(expr, expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_aggregate_root_table_internal(table):
-    pass
+    assert False
 
 
 def test_compound_aggregate_expr(table):
@@ -890,9 +893,10 @@ def test_cross_join_multiple(table):
     assert joined.equals(expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_join_compound_boolean_predicate(table):
     # The user might have composed predicates through logical operations
-    pass
+    assert False
 
 
 def test_filter_join_unmaterialized(table):
@@ -989,12 +993,14 @@ def test_unravel_compound_equijoin(table):
     assert_equal(joined, expected)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_join_add_prefixes(table):
-    pass
+    assert False
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_join_nontrivial_exprs(table):
-    pass
+    assert False
 
 
 def test_union(table):
@@ -1063,7 +1069,7 @@ def t2():
 def test_simple_existence_predicate(t1, t2):
     cond = (t1.key1 == t2.key1).any()
 
-    assert isinstance(cond, ir.BooleanArray)
+    assert isinstance(cond, ir.BooleanColumn)
     op = cond.op()
     assert isinstance(op, ops.Any)
 
@@ -1072,9 +1078,10 @@ def test_simple_existence_predicate(t1, t2):
     assert isinstance(expr.op(), ops.Selection)
 
 
+@pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_cannot_use_existence_expression_in_join(table):
     # Join predicates must consist only of comparisons
-    pass
+    assert False
 
 
 def test_not_exists_predicate(t1, t2):
@@ -1142,7 +1149,7 @@ def test_filter(table):
     assert_equal(result, expected)
 
 
-def test_sort_by(table):
+def test_sort_by2(table):
     m = table.mutate(foo=table.e + table.f)
 
     result = m.sort_by(lambda x: -x.foo)
@@ -1158,7 +1165,7 @@ def test_sort_by(table):
     assert_equal(result, expected)
 
 
-def test_projection(table):
+def test_projection2(table):
     m = table.mutate(foo=table.f * 2)
 
     def f(x):
@@ -1171,7 +1178,7 @@ def test_projection(table):
     assert_equal(result2, expected)
 
 
-def test_mutate(table):
+def test_mutate2(table):
     m = table.mutate(foo=table.f * 2)
 
     def g(x):
@@ -1188,7 +1195,7 @@ def test_mutate(table):
     assert_equal(result, expected)
 
 
-def test_add_column(table):
+def test_add_column2(table):
     def g(x):
         return x.f * 2
 
