@@ -20,6 +20,7 @@ import ibis.common as com
 import ibis.compat as compat
 import ibis.config as config
 import ibis.util as util
+import ibis.expr.datatypes as dt
 
 
 class Parameter(object):
@@ -40,6 +41,9 @@ class Expr(object):
 
     """
 
+    def _type_display(self):
+        return type(self).__name__
+
     def __init__(self, arg):
         # TODO: all inputs must inherit from a common table API
         self._arg = arg
@@ -187,7 +191,26 @@ def _safe_repr(x, memo=None):
     return x._repr(memo=memo) if isinstance(x, (Expr, Node)) else repr(x)
 
 
-class Node(object):
+class OperationMeta(type):
+
+    def __new__(cls, name, parents, dct):
+        if 'input_type' in dct:
+            from ibis.expr.rules import TypeSignature, signature
+            sig = dct['input_type']
+            if not isinstance(sig, TypeSignature):
+                dct['input_type'] = sig = signature(sig)
+
+                for i, t in enumerate(sig.types):
+                    if t.name is None:
+                        continue
+
+                    if t.name not in dct:
+                        dct[t.name] = _arg_getter(i)
+
+        return super(OperationMeta, cls).__new__(cls, name, parents, dct)
+
+
+class Node(six.with_metaclass(OperationMeta, object)):
 
     """
     Node is the base class for all relational algebra and analytical
@@ -202,8 +225,15 @@ class Node(object):
     of Node as merely a typed expression builder.
     """
 
-    def __init__(self, args):
-        self.args = args
+    def __init__(self, args=None):
+        args = args or []
+        self.args = self._validate_args(args)
+
+    def _validate_args(self, args):
+        if not hasattr(self, 'input_type'):
+            return args
+
+        return self.input_type.validate(args)
 
     def __repr__(self):
         return self._repr()
@@ -292,6 +322,15 @@ class Node(object):
         """
         raise NotImplementedError
 
+    @property
+    def _arg_names(self):
+        try:
+            input_type = self.__class__.input_type
+        except AttributeError:
+            return []
+        else:
+            return [t.name for t in getattr(input_type, 'types', [])]
+
 
 def all_equal(left, right, cache=None):
     if isinstance(left, list):
@@ -309,17 +348,17 @@ def all_equal(left, right, cache=None):
     return True
 
 
-class ValueNode(Node):
+def _arg_getter(i):
+    @property
+    def arg_accessor(self):
+        return self.args[i]
+    return arg_accessor
+
+
+class ValueOp(Node):
 
     def __init__(self, *args):
-        args = self._validate_args(args)
-        Node.__init__(self, args)
-
-    def _validate_args(self, args):
-        if not hasattr(self, 'input_type'):
-            return args
-
-        return self.input_type.validate(args)
+        super(ValueOp, self).__init__(args)
 
     def root_tables(self):
         exprs = [arg for arg in self.args if isinstance(arg, Expr)]
@@ -332,7 +371,7 @@ class ValueNode(Node):
         return False
 
 
-class TableColumn(ValueNode):
+class TableColumn(ValueOp):
 
     """
     Selects a column from a TableExpr
@@ -382,6 +421,11 @@ class ExpressionList(Node):
 
 class ExprList(Expr):
 
+    def _type_display(self):
+        list_args = [arg._type_display()
+                     for arg in self.op().args]
+        return ', '.join(list_args)
+
     def exprs(self):
         return self.op().args
 
@@ -414,43 +458,54 @@ class ExprList(Expr):
         return ExpressionList(exprs).to_expr()
 
 
-class Literal(ValueNode):
+class Literal(ValueOp):
 
     def __init__(self, value):
         self.value = value
 
     def __repr__(self):
-        return 'Literal(%s)' % repr(self.value)
+        return '{0}({1})'.format(
+            type(self).__name__,
+            ', '.join(map(repr, self.args))
+        )
 
     @property
     def args(self):
         return [self.value]
 
     def equals(self, other, cache=None):
-        if not isinstance(other, Literal):
-            return False
-        return (isinstance(other.value, type(self.value)) and
-                self.value == other.value)
+        return (
+            isinstance(other, Literal) and
+            isinstance(other.value, type(self.value)) and
+            self.value == other.value
+        )
 
     def output_type(self):
         import ibis.expr.rules as rules
-        if isinstance(self.value, bool):
-            klass = BooleanScalar
-        elif isinstance(self.value, compat.integer_types):
-            int_type = rules.int_literal_class(self.value)
-            klass = int_type.scalar_type()
-        elif isinstance(self.value, float):
-            klass = DoubleScalar
-        elif isinstance(self.value, six.string_types):
-            klass = StringScalar
-        elif isinstance(self.value, datetime.datetime):
-            klass = TimestampScalar
-        elif isinstance(self.value, datetime.date):
-            klass = DateScalar
-        else:
-            raise com.InputTypeError(self.value)
 
-        return klass
+        value = self.value
+
+        if isinstance(value, bool):
+            return BooleanScalar
+        elif isinstance(value, compat.integer_types):
+            return rules.int_literal_class(value).scalar_type()
+        elif isinstance(value, float):
+            return DoubleScalar
+        elif isinstance(value, six.string_types):
+            return StringScalar
+        elif isinstance(value, datetime.datetime):
+            return TimestampScalar
+        elif isinstance(value, datetime.date):
+            return DateScalar
+        elif isinstance(value, list):
+            value_type = rules.highest_precedence_type(
+                list(map(literal, value))
+            )
+            return lambda value, value_type=value_type: ArrayScalar(
+                value, dt.Array(value_type)
+            )
+
+        raise com.InputTypeError(value)
 
     def root_tables(self):
         return []
@@ -474,28 +529,25 @@ class ValueExpr(Expr):
     either a single value (scalar)
     """
 
-    _implicit_casts = set()
+    _implicit_casts = frozenset()
 
     def __init__(self, arg, name=None):
-        Expr.__init__(self, arg)
+        super(ValueExpr, self).__init__(arg)
         self._name = name
 
     def equals(self, other, cache=None):
-        if not isinstance(other, ValueExpr):
-            return False
-
-        if self._name != other._name:
-            return False
-
-        return Expr.equals(self, other, cache=cache)
+        return (
+            isinstance(other, ValueExpr) and
+            self._name == other._name and
+            super(ValueExpr, self).equals(other, cache=cache)
+        )
 
     def type(self):
-        import ibis.expr.datatypes as dt
-        return dt._primitive_types[self._typename]
-
-    def _base_type(self):
-        # Parametric types like "decimal"
-        return self.type()
+        raise NotImplementedError(
+            'Expressions of type {0} must implement a type method'.format(
+                type(self).__name__
+            )
+        )
 
     def _can_cast_implicit(self, typename):
         from ibis.expr.rules import ImplicitCast
@@ -522,10 +574,14 @@ class ValueExpr(Expr):
 
 class ScalarExpr(ValueExpr):
 
-    pass
+    def _type_display(self):
+        return str(self.type())
 
 
-class ArrayExpr(ValueExpr):
+class ColumnExpr(ValueExpr):
+
+    def _type_display(self):
+        return '{}*'.format(self.type())
 
     def parent(self):
         return self._arg
@@ -552,6 +608,9 @@ class AnalyticExpr(Expr):
             return type(self)(arg)
         return factory
 
+    def _type_display(self):
+        return str(self.type())
+
     def type(self):
         return 'analytic'
 
@@ -564,6 +623,9 @@ class TableExpr(Expr):
             return TableExpr(arg)
         return factory
 
+    def _type_display(self):
+        return 'table'
+
     def _is_valid(self, exprs):
         try:
             self._assert_valid(util.promote_list(exprs))
@@ -605,10 +667,10 @@ class TableExpr(Expr):
         if isinstance(what, (list, tuple, TableExpr)):
             # Projection case
             return self.projection(what)
-        elif isinstance(what, BooleanArray):
+        elif isinstance(what, BooleanColumn):
             # Boolean predicate
             return self.filter([what])
-        elif isinstance(what, ArrayExpr):
+        elif isinstance(what, ColumnExpr):
             # Projection convenience
             return self.projection(what)
         else:
@@ -709,7 +771,7 @@ class TableExpr(Expr):
         """
         expr = self._ensure_expr(expr)
 
-        if not isinstance(expr, ArrayExpr):
+        if not isinstance(expr, ColumnExpr):
             raise com.InputTypeError('Must pass array expression')
 
         if name is not None:
@@ -749,12 +811,14 @@ class TableExpr(Expr):
 
 class AnyValue(ValueExpr):
 
-    _typename = 'any'
+    def type(self):
+        return dt.any
 
 
 class NullValue(AnyValue):
 
-    _typename = 'null'
+    def type(self):
+        return dt.null
 
     def _can_cast_implicit(self, typename):
         return True
@@ -772,33 +836,43 @@ class IntegerValue(NumericValue):
 
 class BooleanValue(NumericValue):
 
-    _typename = 'boolean'
+    def type(self):
+        return dt.boolean
 
 
 class Int8Value(IntegerValue):
 
-    _typename = 'int8'
-    _implicit_casts = set(['int16', 'int32', 'int64', 'float', 'double',
-                           'decimal'])
+    _implicit_casts = set([
+        'int16', 'int32', 'int64', 'float', 'double', 'decimal'
+    ])
+
+    def type(self):
+        return dt.int8
 
 
 class Int16Value(IntegerValue):
 
-    _typename = 'int16'
     _implicit_casts = set(['int32', 'int64', 'float', 'double', 'decimal'])
 
+    def type(self):
+        return dt.int16
+
 
 class Int32Value(IntegerValue):
 
-    _typename = 'int32'
     _implicit_casts = set(['int64', 'float', 'double', 'decimal'])
 
+    def type(self):
+        return dt.int32
+
 
 class Int64Value(IntegerValue):
 
-    _typename = 'int64'
     _implicit_casts = set(['float', 'double', 'decimal'])
 
+    def type(self):
+        return dt.int64
+
 
 class FloatingValue(NumericValue):
     pass
@@ -806,19 +880,24 @@ class FloatingValue(NumericValue):
 
 class FloatValue(FloatingValue):
 
-    _typename = 'float'
     _implicit_casts = set(['double', 'decimal'])
 
+    def type(self):
+        return dt.float
+
 
 class DoubleValue(FloatingValue):
 
-    _typename = 'double'
     _implicit_casts = set(['decimal'])
 
+    def type(self):
+        return dt.double
+
 
 class StringValue(AnyValue):
 
-    _typename = 'string'
+    def type(self):
+        return dt.string
 
     def _can_compare(self, other):
         return isinstance(other, StringValue)
@@ -826,7 +905,6 @@ class StringValue(AnyValue):
 
 class DecimalValue(NumericValue):
 
-    _typename = 'decimal'
     _implicit_casts = set(['float', 'double'])
 
     def __init__(self, meta):
@@ -835,11 +913,7 @@ class DecimalValue(NumericValue):
         self._scale = meta.scale
 
     def type(self):
-        from ibis.expr.datatypes import Decimal
-        return Decimal(self._precision, self._scale)
-
-    def _base_type(self):
-        return 'decimal'
+        return dt.Decimal(self._precision, self._scale)
 
     @classmethod
     def _make_constructor(cls, meta):
@@ -850,7 +924,8 @@ class DecimalValue(NumericValue):
 
 class DateValue(AnyValue):
 
-    _typename = 'date'
+    def type(self):
+        return dt.date
 
     def _can_implicit_cast(self, arg):
         op = arg.op()
@@ -874,7 +949,8 @@ class DateValue(AnyValue):
 
 class TimestampValue(AnyValue):
 
-    _typename = 'timestamp'
+    def type(self):
+        return dt.timestamp
 
     def _can_implicit_cast(self, arg):
         op = arg.op()
@@ -896,7 +972,20 @@ class TimestampValue(AnyValue):
         return TimestampScalar(op)
 
 
-class NumericArray(ArrayExpr, NumericValue):
+class ArrayValue(AnyValue):
+
+    def __init__(self, type, name=None):
+        super(ArrayValue, self).__init__(type.value_type)
+        self.value_type = type.value_type
+
+    def type(self):
+        return dt.Array(self.value_type)
+
+    def _can_compare(self, other):
+        return isinstance(other, ArrayValue)
+
+
+class NumericColumn(ColumnExpr, NumericValue):
     pass
 
 
@@ -907,7 +996,7 @@ class NullScalar(NullValue, ScalarExpr):
     pass
 
 
-class NullArray(ArrayExpr, NullValue):
+class NullColumn(ColumnExpr, NullValue):
     pass
 
 
@@ -915,7 +1004,7 @@ class BooleanScalar(ScalarExpr, BooleanValue):
     pass
 
 
-class BooleanArray(NumericArray, BooleanValue):
+class BooleanColumn(NumericColumn, BooleanValue):
     pass
 
 
@@ -923,7 +1012,7 @@ class Int8Scalar(ScalarExpr, Int8Value):
     pass
 
 
-class Int8Array(NumericArray, Int8Value):
+class Int8Column(NumericColumn, Int8Value):
     pass
 
 
@@ -931,7 +1020,7 @@ class Int16Scalar(ScalarExpr, Int16Value):
     pass
 
 
-class Int16Array(NumericArray, Int16Value):
+class Int16Column(NumericColumn, Int16Value):
     pass
 
 
@@ -939,7 +1028,7 @@ class Int32Scalar(ScalarExpr, Int32Value):
     pass
 
 
-class Int32Array(NumericArray, Int32Value):
+class Int32Column(NumericColumn, Int32Value):
     pass
 
 
@@ -947,7 +1036,7 @@ class Int64Scalar(ScalarExpr, Int64Value):
     pass
 
 
-class Int64Array(NumericArray, Int64Value):
+class Int64Column(NumericColumn, Int64Value):
     pass
 
 
@@ -955,7 +1044,7 @@ class FloatScalar(ScalarExpr, FloatValue):
     pass
 
 
-class FloatArray(NumericArray, FloatValue):
+class FloatColumn(NumericColumn, FloatValue):
     pass
 
 
@@ -963,7 +1052,7 @@ class DoubleScalar(ScalarExpr, DoubleValue):
     pass
 
 
-class DoubleArray(NumericArray, DoubleValue):
+class DoubleColumn(NumericColumn, DoubleValue):
     pass
 
 
@@ -971,7 +1060,7 @@ class StringScalar(ScalarExpr, StringValue):
     pass
 
 
-class StringArray(ArrayExpr, StringValue):
+class StringColumn(ColumnExpr, StringValue):
     pass
 
 
@@ -979,7 +1068,7 @@ class DateScalar(ScalarExpr, DateValue):
     pass
 
 
-class DateArray(ArrayExpr, DateValue):
+class DateColumn(ColumnExpr, DateValue):
     pass
 
 
@@ -987,7 +1076,7 @@ class TimestampScalar(ScalarExpr, TimestampValue):
     pass
 
 
-class TimestampArray(ArrayExpr, TimestampValue):
+class TimestampColumn(ColumnExpr, TimestampValue):
     pass
 
 
@@ -1004,16 +1093,16 @@ class DecimalScalar(DecimalValue, ScalarExpr):
         return factory
 
 
-class DecimalArray(DecimalValue, NumericArray):
+class DecimalColumn(DecimalValue, NumericColumn):
 
     def __init__(self, arg, meta, name=None):
         DecimalValue.__init__(self, meta)
-        ArrayExpr.__init__(self, arg, name=name)
+        NumericColumn.__init__(self, arg, name=name)
 
     @property
     def _factory(self):
         def factory(arg, name=None):
-            return DecimalArray(arg, self.meta, name=name)
+            return DecimalColumn(arg, self.meta, name=name)
         return factory
 
 
@@ -1024,7 +1113,6 @@ class CategoryValue(AnyValue):
     until explicitly
     """
 
-    _typename = 'category'
     _implicit_casts = Int16Value._implicit_casts
 
     def __init__(self, meta):
@@ -1033,9 +1121,6 @@ class CategoryValue(AnyValue):
     def type(self):
         return self.meta
 
-    def _base_type(self):
-        return 'category'
-
     def _can_compare(self, other):
         return isinstance(other, IntegerValue)
 
@@ -1053,16 +1138,42 @@ class CategoryScalar(CategoryValue, ScalarExpr):
         return factory
 
 
-class CategoryArray(CategoryValue, ArrayExpr):
+class CategoryColumn(CategoryValue, ColumnExpr):
 
     def __init__(self, arg, meta, name=None):
         CategoryValue.__init__(self, meta)
-        ArrayExpr.__init__(self, arg, name=name)
+        ColumnExpr.__init__(self, arg, name=name)
 
     @property
     def _factory(self):
         def factory(arg, name=None):
-            return CategoryArray(arg, self.meta, name=name)
+            return CategoryColumn(arg, self.meta, name=name)
+        return factory
+
+
+class ArrayScalar(ArrayValue, ScalarExpr):
+
+    def __init__(self, arg, meta, name=None):
+        ArrayValue.__init__(self, meta)
+        ScalarExpr.__init__(self, arg, name=name)
+
+    @property
+    def _factory(self):
+        def factory(arg, name=None):
+            return ArrayScalar(arg, self.type(), name=name)
+        return factory
+
+
+class ArrayColumn(ArrayValue, ColumnExpr):
+
+    def __init__(self, arg, meta, name=None):
+        ArrayValue.__init__(self, meta)
+        ColumnExpr.__init__(self, arg, name=name)
+
+    @property
+    def _factory(self):
+        def factory(arg, name=None):
+            return ArrayColumn(arg, self.type(), name=name)
         return factory
 
 
@@ -1134,7 +1245,7 @@ def sequence(values):
     return ValueList(values).to_expr()
 
 
-class NullLiteral(ValueNode):
+class NullLiteral(ValueOp):
 
     """
     Typeless NULL literal
@@ -1157,15 +1268,17 @@ class NullLiteral(ValueNode):
         return []
 
 
-class ListExpr(ArrayExpr, AnyValue):
+class ListExpr(ColumnExpr, AnyValue):
     pass
 
 
 class SortExpr(Expr):
-    pass
+
+    def _type_display(self):
+        return 'array-sort'
 
 
-class ValueList(ValueNode):
+class ValueList(ValueOp):
 
     """
     Data structure for a list of value expressions
@@ -1173,7 +1286,7 @@ class ValueList(ValueNode):
 
     def __init__(self, args):
         self.values = [as_value_expr(x) for x in args]
-        ValueNode.__init__(self, self.values)
+        ValueOp.__init__(self, self.values)
 
     def root_tables(self):
         return distinct_roots(*self.values)
