@@ -12,12 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from posixpath import join as pjoin
 import re
 import six
 import threading
 import time
 import weakref
+import traceback
+
+from posixpath import join as pjoin
+from collections import deque
 
 import hdfs
 import numpy as np
@@ -41,12 +44,6 @@ import ibis.expr.types as ir
 import ibis.util as util
 
 
-if six.PY2:
-    import Queue as queue
-else:
-    import queue
-
-
 class ImpalaDatabase(Database):
 
     def create_table(self, table_name, obj=None, **kwargs):
@@ -81,9 +78,11 @@ class ImpalaConnection(object):
         self.options = {}
 
         self.max_pool_size = pool_size
-        self._connections = None
+        self._connections = weakref.WeakSet()
+
+        self.connection_pool = deque(maxlen=pool_size)
+        self.connection_pool_size = 0
 
-        self.reset_connection_pool()
         self.ping()
 
     def set_options(self, options):
@@ -93,15 +92,11 @@ class ImpalaConnection(object):
         """
         Close all open Impyla sessions
         """
-        self.reset_connection_pool()
+        for impyla_connection in self._connections:
+            impyla_connection.close()
 
-    def reset_connection_pool(self):
-        if self._connections is not None:
-            for k, con in self._connections.items():
-                con.close()
-        self._connections = weakref.WeakValueDictionary()
-        self.connection_pool = queue.Queue(self.max_pool_size)
-        self.connection_pool_size = 0
+        self._connections.clear()
+        self.connection_pool.clear()
 
     def set_database(self, name):
         self.database = name
@@ -123,13 +118,9 @@ class ImpalaConnection(object):
         try:
             cursor.execute(query, async=async)
         except:
+            exc = traceback.format_exc()
             cursor.release()
-
-            import traceback
-            buf = six.StringIO()
-            traceback.print_exc(file=buf)
-            self.error('Exception caused by {0}: {1}'.format(query,
-                                                             buf.getvalue()))
+            self.error('Exception caused by {}: {}'.format(query, exc))
             raise
 
         return cursor
@@ -147,26 +138,23 @@ class ImpalaConnection(object):
 
     def _get_cursor(self):
         try:
-            cur = self.connection_pool.get(False)
-            if (cur.database != self.database or
-                    cur.options != self.options):
-                cur = self._new_cursor()
-            cur.released = False
-
-            return cur
-        except queue.Empty:
+            cursor = self.connection_pool.popleft()
+        except IndexError:  # deque is empty
             if self.connection_pool_size < self.max_pool_size:
-                cursor = self._new_cursor()
-                self.connection_pool_size += 1
-                return cursor
-            else:
-                raise com.InternalError('Too many concurrent / hung queries')
+                return self._new_cursor()
+            raise com.InternalError('Too many concurrent / hung queries')
+        else:
+            if (cursor.database != self.database or
+                    cursor.options != self.options):
+                return self._new_cursor()
+            cursor.released = False
+            return cursor
 
     def _new_cursor(self):
         params = self.params.copy()
         con = impyla.connect(database=self.database, **params)
 
-        self._connections[id(con)] = con
+        self._connections.add(con)
 
         # make sure the connection works
         cursor = con.cursor(convert_types=True)
@@ -179,10 +167,10 @@ class ImpalaConnection(object):
         return wrapper
 
     def ping(self):
-        self._new_cursor()
+        self._get_cursor()._cursor.ping()
 
     def release(self, cur):
-        self.connection_pool.put(cur)
+        self.connection_pool.append(cur)
 
 
 class ImpalaCursor(object):
@@ -195,6 +183,7 @@ class ImpalaCursor(object):
         self.database = database
         self.options = options
         self.released = False
+        self.con.connection_pool_size += 1
 
     def __del__(self):
         self._close_cursor()
@@ -1647,6 +1636,9 @@ class ImpalaTable(ir.TableExpr, DatabaseEntity):
         """
         self._client.drop_table_or_view(self._qualified_name)
 
+    def truncate(self):
+        self._client.truncate_table(self._qualified_name)
+
     def insert(self, obj=None, overwrite=False, partition=None,
                values=None, validate=True):
         """
@@ -1691,7 +1683,11 @@ class ImpalaTable(ir.TableExpr, DatabaseEntity):
 
         if partition is not None:
             partition_schema = self.partition_schema()
-            expr = expr.drop(partition_schema.names)
+            partition_schema_names = frozenset(partition_schema.names)
+            expr = expr.projection([
+                column for column in expr.columns
+                if column not in partition_schema_names
+            ])
         else:
             partition_schema = None
 
@@ -1956,6 +1952,7 @@ def _split_signature(x):
     name, rest = x.split('(', 1)
     return name, rest[:-1]
 
+
 _arg_type = re.compile('(.*)\.\.\.|([^\.]*)')
 
 
