@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import numpy as np
 import pandas as pd
 
 from ibis.compat import unittest
@@ -24,6 +25,11 @@ import ibis.config as config
 import ibis.expr.types as ir
 import ibis.util as util
 
+import pytest
+
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
 
 ENV = IbisTestEnv()
 
@@ -281,8 +287,12 @@ LIMIT 10"""
         q = expr.execute(async=True)
         result = q.get_result()
         expected = expr.execute()
-        assert result == expected
+        assert np.allclose(result, expected)
 
+    @pytest.mark.xfail(
+        raises=NotImplementedError,
+        reason='_collect_Union not implemented'
+    )
     def test_query_cancel(self):
         import time
         t = self.db.functional_alltypes
