@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# flake8: noqa=E402
+
 from copy import copy
 import gc
 
@@ -21,6 +23,9 @@ import pandas as pd
 from posixpath import join as pjoin
 import pytest
 
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
 from ibis.expr.tests.mocks import MockConnection
 from ibis.compat import unittest, mock
 from ibis.impala import ddl
@@ -116,8 +121,9 @@ LOAD DATA INPATH '/path/to/data' OVERWRITE INTO TABLE foo.`functional_alltypes`
 PARTITION (year=2007, month=7)"""
         assert result == expected
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_select_overwrite(self):
-        pass
+        assert False
 
 
 class TestCacheTable(unittest.TestCase):
@@ -500,8 +506,9 @@ FROM functional_alltypes"""
         self.assertRaises(ValueError, _create_table, 'tname', self.t,
                           format='foo')
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_partition_by(self):
-        pass
+        assert False
 
 
 class TestDDLE2E(ImpalaE2E, unittest.TestCase):
@@ -677,6 +684,16 @@ class TestDDLE2E(ImpalaE2E, unittest.TestCase):
         result = self.con.table(table_name).execute()
         assert len(result) == 0
 
+    def test_truncate_table_expression(self):
+        expr = self.alltypes.limit(5)
+
+        table_name = _random_table_name()
+        self.con.create_table(table_name, obj=expr)
+        self.temp_tables.append(table_name)
+        t = self.con.table(table_name)
+        t.truncate()
+        assert len(t.execute()) == 0
+
     def test_ctas_from_table_expr(self):
         expr = self.alltypes
         table_name = _random_table_name()
@@ -1022,8 +1039,9 @@ class TestDDLE2E(ImpalaE2E, unittest.TestCase):
         t = self.con.table(table_name)
         t.limit(10).execute()
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_query_text_file_regex(self):
-        pass
+        assert False
 
     def test_query_delimited_file_directory(self):
         hdfs_path = pjoin(self.test_data_dir, 'csv')
