@@ -12,9 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# flake8: noqa=E402
+
 import pytest
 
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
 import pandas as pd
+import pandas.util.testing as tm
 
 import ibis
 
@@ -379,7 +385,7 @@ class TestUnaryBuiltins(unittest.TestCase, ExprSQLTest):
 
     def test_hash(self):
         expr = self.table.int_col.hash()
-        assert isinstance(expr, ir.Int64Array)
+        assert isinstance(expr, ir.Int64Column)
         assert isinstance(self.table.int_col.sum().hash(),
                           ir.Int64Scalar)
 
@@ -484,6 +490,21 @@ END"""
         ]
         self._check_expr_cases(cases)
 
+    def test_identical_to(self):
+        t = self.con.table('functional_alltypes')
+        expr = t.tinyint_col.identical_to(t.double_col)
+        result = to_sql(expr)
+        expected = """\
+SELECT `tinyint_col` IS NOT DISTINCT FROM `double_col` AS `tmp`
+FROM functional_alltypes"""
+        assert result == expected
+
+    def test_identical_to_special_case(self):
+        expr = ibis.NA.cast('int64').identical_to(ibis.NA.cast('int64'))
+        result = to_sql(expr)
+        assert result == 'SELECT TRUE AS `tmp`'
+
+
 
 class TestBucketHistogram(unittest.TestCase, ExprSQLTest):
 
@@ -782,6 +803,8 @@ class TestAnalyticFunctions(unittest.TestCase, ExprSQLTest):
             (t.double_col.first(), 'first_value(`double_col`)'),
             (t.double_col.last(), 'last_value(`double_col`)'),
             # (t.double_col.nth(4), 'first_value(lag(double_col, 4 - 1))')
+            (t.double_col.ntile(3), 'ntile(3)'),
+            (t.double_col.percent_rank(), 'percent_rank()'),
         ]
         self._check_expr_cases(cases)
 
@@ -1438,6 +1461,8 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
             f.lead(),
             f.rank(),
             f.dense_rank(),
+            f.percent_rank(),
+            f.ntile(buckets=7),
 
             f.first(),
             f.last(),
@@ -1548,8 +1573,8 @@ FROM functional_alltypes"""
 
         t = self.con.sql(sql)
 
-        assert isinstance(t.varchar_col, api.StringArray)
-        assert isinstance(t.char_col, api.StringArray)
+        assert isinstance(t.varchar_col, api.StringColumn)
+        assert isinstance(t.char_col, api.StringColumn)
 
     def test_unions_with_ctes(self):
         t = self.con.table('functional_alltypes')
@@ -1565,6 +1590,34 @@ FROM functional_alltypes"""
         expr = join1.union(join2)
         self.con.explain(expr)
 
+    def test_head(self):
+        t = self.con.table('functional_alltypes')
+        result = t.head().execute()
+        expected = t.limit(5).execute()
+        tm.assert_frame_equal(result, expected)
+
+    def test_identical_to(self):
+        cases = [
+            (ibis.NA.cast('int64'), ibis.NA.cast('int64'), True),
+            (L(1), L(1), True),
+            (ibis.NA.cast('int64'), L(1), False),
+            (L(1), ibis.NA.cast('int64'), False),
+            (L(0), L(1), False),
+            (L(1), L(0), False),
+        ]
+        con = self.con
+        for left, right, expected in cases:
+            expr = left.identical_to(right)
+            result = con.execute(expr)
+            assert result == expected
+
+    def test_not(self):
+        t = self.con.table('functional_alltypes').limit(10)
+        expr = t.projection([(~t.double_col.isnull()).name('double_col')])
+        result = expr.execute().double_col
+        expected = ~t.execute().double_col.isnull()
+        tm.assert_series_equal(result, expected)
+
 
 def test_where_with_timestamp():
     t = ibis.table(
@@ -1585,3 +1638,33 @@ SELECT `uuid`,
 FROM t
 GROUP BY 1"""
     assert result == expected
+
+
+def test_filter_with_analytic():
+    x = ibis.table(ibis.schema([('col', 'int32')]), 'x')
+    with_filter_col = x[x.columns + [ibis.null().name('filter')]]
+    filtered = with_filter_col[with_filter_col['filter'].isnull()]
+    subquery = filtered[filtered.columns]
+
+    with_analytic = subquery[['col', subquery.count().name('analytic')]]
+    expr = with_analytic[with_analytic.columns]
+
+    result = ibis.impala.compile(expr)
+    expected = """\
+SELECT `col`, `analytic`
+FROM (
+  SELECT `col`, count(*) OVER () AS `analytic`
+  FROM (
+    SELECT `col`, `filter`
+    FROM (
+      SELECT *
+      FROM (
+        SELECT `col`, NULL AS `filter`
+        FROM x
+      ) t3
+      WHERE `filter` IS NULL
+    ) t2
+  ) t1
+) t0"""
+
+    assert result == expected
