@@ -11,9 +11,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# flake8: noqa=E402
+
 from posixpath import join as pjoin
 import pytest
 
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
 import ibis
 
 import ibis.expr.types as ir
@@ -116,7 +121,7 @@ class TestWrapping(unittest.TestCase):
         expr = func(1.0)
         assert type(expr) == ir.DecimalScalar
         expr = func(self.dec)
-        assert type(expr) == ir.DecimalArray
+        assert type(expr) == ir.DecimalColumn
 
     def test_udf_invalid_typecasting(self):
         cases = [
@@ -150,7 +155,7 @@ class TestWrapping(unittest.TestCase):
                                   'int64', 'mult_types')
 
         expr = func(self.i32, self.d, self.s, self.b, self.t)
-        assert issubclass(type(expr), ir.ArrayExpr)
+        assert issubclass(type(expr), ir.ColumnExpr)
 
         expr = func(1, 1.0, 'a', True, ibis.timestamp('1961-04-10'))
         assert issubclass(type(expr), ir.ScalarExpr)
@@ -173,6 +178,11 @@ class TestUDFE2E(ImpalaE2E, unittest.TestCase):
         self.udf_ll = pjoin(self.test_data_dir, 'udf/udf-sample.ll')
         self.uda_ll = pjoin(self.test_data_dir, 'udf/uda-sample.ll')
         self.uda_so = pjoin(self.test_data_dir, 'udf/libudasample.so')
+        self.con.disable_codegen(False)
+
+    def tearDown(self):
+        super(TestUDFE2E, self).tearDown()
+        self.con.disable_codegen(True)
 
     @pytest.mark.udf
     def test_identity_primitive_types(self):
@@ -209,7 +219,7 @@ class TestUDFE2E(ImpalaE2E, unittest.TestCase):
         assert result == Decimal(1)
 
         expr = func(col)
-        assert issubclass(type(expr), ir.ArrayExpr)
+        assert issubclass(type(expr), ir.ColumnExpr)
         self.con.execute(expr)
 
     @pytest.mark.udf
@@ -221,11 +231,11 @@ class TestUDFE2E(ImpalaE2E, unittest.TestCase):
         func = self._udf_creation_to_op(name, symbol, inputs, output)
 
         expr = func(self.alltypes.int_col, 1)
-        assert issubclass(type(expr), ir.ArrayExpr)
+        assert issubclass(type(expr), ir.ColumnExpr)
         self.con.execute(expr)
 
         expr = func(1, self.alltypes.int_col)
-        assert issubclass(type(expr), ir.ArrayExpr)
+        assert issubclass(type(expr), ir.ColumnExpr)
         self.con.execute(expr)
 
         expr = func(self.alltypes.int_col, self.alltypes.tinyint_col)
@@ -257,7 +267,7 @@ class TestUDFE2E(ImpalaE2E, unittest.TestCase):
                 self.assertAlmostEqual(result, self.con.execute(literal), 5)
 
         expr = func(column)
-        assert issubclass(type(expr), ir.ArrayExpr)
+        assert issubclass(type(expr), ir.ColumnExpr)
         self.con.execute(expr)
 
     @pytest.mark.udf
