@@ -46,7 +46,7 @@ _ibis_type_to_sqla = {
 
     dt.Boolean: sa.Boolean,
 
-    dt.String: sa.String,
+    dt.String: sa.Text,
 
     dt.Date: sa.Date,
 
@@ -64,53 +64,68 @@ _sqla_type_mapping = {
     sa.BIGINT: dt.Int64,
     sa.Boolean: dt.Boolean,
     sa.BOOLEAN: dt.Boolean,
+    sa.Float: dt.Double,
     sa.FLOAT: dt.Double,
     sa.REAL: dt.Float,
+    sa.String: dt.String,
     sa.VARCHAR: dt.String,
-    sa.Float: dt.Double,
+    sa.CHAR: dt.String,
+    sa.Text: dt.String,
+    sa.TEXT: dt.String,
     sa.DATE: dt.Date,
-
-    sa.types.TEXT: dt.String,
     sa.types.NullType: dt.Null,
-    sa.types.Text: dt.String,
 }
 
-_sqla_type_to_ibis = dict((v, k) for k, v in
-                          _ibis_type_to_sqla.items())
+_sqla_type_to_ibis = dict((v, k) for k, v in _ibis_type_to_sqla.items())
 _sqla_type_to_ibis.update(_sqla_type_mapping)
 
 
+def sqlalchemy_type_to_ibis_type(column_type, nullable=True):
+    type_class = type(column_type)
+
+    if isinstance(column_type, sa.types.NUMERIC):
+        return dt.Decimal(
+            column_type.precision, column_type.scale, nullable=nullable
+        )
+    else:
+        if type_class in _sqla_type_to_ibis:
+            ibis_class = _sqla_type_to_ibis[type_class]
+        elif isinstance(column_type, sa.DateTime):
+            ibis_class = dt.Timestamp()
+        elif isinstance(column_type, sa.ARRAY):
+            dimensions = column_type.dimensions
+            if dimensions is not None and dimensions != 1:
+                raise NotImplementedError(
+                    'Nested array types not yet supported'
+                )
+            value_type = sqlalchemy_type_to_ibis_type(column_type.item_type)
+
+            def make_array_type(nullable, value_type=value_type):
+                return dt.Array(value_type, nullable=nullable)
+
+            ibis_class = make_array_type
+        else:
+            try:
+                ibis_class = next(
+                    v for k, v in _sqla_type_mapping.items()
+                    if isinstance(column_type, k)
+                )
+            except StopIteration:
+                raise NotImplementedError(
+                    'Unable to convert SQLAlchemy type {} to ibis type'.format(
+                        column_type
+                    )
+                )
+        return ibis_class(nullable)
+
+
 def schema_from_table(table):
     # Convert SQLA table to Ibis schema
-    names = table.columns.keys()
-
-    types = []
-    for c in table.columns.values():
-        type_class = type(c.type)
-
-        if isinstance(c.type, sa.types.NUMERIC):
-            t = dt.Decimal(c.type.precision,
-                           c.type.scale,
-                           nullable=c.nullable)
-        else:
-            if c.type in _sqla_type_to_ibis:
-                ibis_class = _sqla_type_to_ibis[c.type]
-            elif type_class in _sqla_type_to_ibis:
-                ibis_class = _sqla_type_to_ibis[type_class]
-            elif isinstance(c.type, sa.DateTime):
-                ibis_class = dt.Timestamp()
-            else:
-                for k, v in _sqla_type_to_ibis.items():
-                    if isinstance(c.type, type(k)):
-                        ibis_class = v
-                        break
-                else:
-                    raise NotImplementedError(c.type)
-            t = ibis_class(c.nullable)
-
-        types.append(t)
-
-    return dt.Schema(names, types)
+    types = [
+        sqlalchemy_type_to_ibis_type(column.type, column.nullable)
+        for column in table.columns.values()
+    ]
+    return dt.Schema(table.columns.keys(), types)
 
 
 def table_from_schema(name, meta, schema):
@@ -126,11 +141,22 @@ def table_from_schema(name, meta, schema):
     return sa.Table(name, meta, *sqla_cols)
 
 
-def _to_sqla_type(itype):
+def _to_sqla_type(itype, type_map=None):
+    if type_map is None:
+        type_map = _ibis_type_to_sqla
     if isinstance(itype, dt.Decimal):
         return sa.types.NUMERIC(itype.precision, itype.scale)
+    elif isinstance(itype, dt.Array):
+        ibis_type = itype.value_type
+        if not isinstance(ibis_type, (dt.Primitive, dt.String)):
+            raise TypeError(
+                'Type {} is not a primitive type or string type'.format(
+                    ibis_type
+                )
+            )
+        return sa.ARRAY(_to_sqla_type(ibis_type, type_map=type_map))
     else:
-        return _ibis_type_to_sqla[type(itype)]
+        return type_map[type(itype)]
 
 
 def fixed_arity(sa_func, arity):
@@ -315,6 +341,12 @@ def _translate_case(t, cases, results, default):
     return sa.case(whens, else_=default)
 
 
+def _negate(t, expr):
+    op = expr.op()
+    arg, = map(t.translate, op.args)
+    return sa.not_(arg) if isinstance(expr, ir.BooleanValue) else -arg
+
+
 def unary(sa_func):
     return fixed_arity(sa_func, 1)
 
@@ -322,6 +354,7 @@ def unary(sa_func):
 _operation_registry = {
     ops.And: fixed_arity(sql.and_, 2),
     ops.Or: fixed_arity(sql.or_, 2),
+    ops.Not: unary(sa.not_),
 
     ops.Abs: unary(sa.func.abs),
 
@@ -347,7 +380,7 @@ _operation_registry = {
 
     ops.IsNull: _is_null,
     ops.NotNull: _not_null,
-    ops.Negate: unary(sa.not_),
+    ops.Negate: _negate,
 
     ops.Round: _round,
 
@@ -385,6 +418,7 @@ _binary_ops = {
     ops.LessEqual: operator.le,
     ops.Greater: operator.gt,
     ops.GreaterEqual: operator.ge,
+    ops.IdenticalTo: lambda x, y: x.op('IS NOT DISTINCT FROM')(y),
 
     # Boolean comparisons
 
@@ -412,17 +446,14 @@ class AlchemyContext(comp.QueryContext):
         self.dialect = kwargs.pop('dialect', AlchemyDialect)
         comp.QueryContext.__init__(self, *args, **kwargs)
 
-    def subcontext(self, isolated=False):
-        if not isolated:
-            return type(self)(dialect=self.dialect, parent=self)
-        else:
-            return type(self)(dialect=self.dialect)
+    def subcontext(self):
+        return type(self)(dialect=self.dialect, parent=self)
 
     def _to_sql(self, expr, ctx):
         return to_sqlalchemy(expr, context=ctx)
 
-    def _compile_subquery(self, expr, isolated=False):
-        sub_ctx = self.subcontext(isolated=isolated)
+    def _compile_subquery(self, expr):
+        sub_ctx = self.subcontext()
         return self._to_sql(expr, sub_ctx)
 
     def has_table(self, expr, parent_contexts=False):
@@ -504,7 +535,7 @@ class AlchemyExprTranslator(comp.ExprTranslator):
         return AlchemyContext
 
     def get_sqla_type(self, data_type):
-        return self._type_map[type(data_type)]
+        return _to_sqla_type(data_type, type_map=self._type_map)
 
 
 rewrites = AlchemyExprTranslator.rewrites
@@ -542,7 +573,45 @@ class AlchemyClient(SQLClient):
         raise NotImplementedError
 
     def create_table(self, name, expr=None, schema=None, database=None):
-        pass
+        if database is not None and database != self.engine.url.database:
+            raise NotImplementedError(
+                'Creating tables from a different database is not yet '
+                'implemented'
+            )
+
+        if expr is None and schema is None:
+            raise ValueError('You must pass either an expression or a schema')
+
+        if expr is not None and schema is not None:
+            if not expr.schema().equals(ibis.schema(schema)):
+                raise TypeError(
+                    'Expression schema is not equal to passed schema. '
+                    'Try passing the expression without the schema'
+                )
+        t = table_from_schema(name, self.meta, schema or expr.schema())
+        with self.con.begin() as bind:
+            t.create(bind=bind)
+            if expr is not None:
+                bind.execute(
+                    t.insert().from_select(list(expr.columns), expr.compile())
+                )
+
+    def drop_table(self, table_name, database=None, force=False):
+        if database is not None and database != self.engine.url.database:
+            raise NotImplementedError(
+                'Dropping tables from a different database is not yet '
+                'implemented'
+            )
+        t = self.meta.tables[table_name]
+        t.drop(checkfirst=force)
+        if t.exists():
+            raise RuntimeError(
+                'Something went wrong during DROP of table {}'.format(t.name)
+            )
+        self.meta.remove(t)
+
+    def truncate_table(self, table_name, database=None):
+        self.meta.tables[table_name].delete().execute()
 
     def list_tables(self, like=None, database=None):
         """
@@ -772,6 +841,14 @@ class _AlchemyTableSet(TableSetFormatter):
                 result = table.join(result, onclause, isouter=True)
             elif jtype is ops.OuterJoin:
                 result = result.outerjoin(table, onclause)
+            elif jtype is ops.LeftSemiJoin:
+                result = (sa.select([result])
+                          .where(sa.exists(sa.select([1])
+                                           .where(onclause))))
+            elif jtype is ops.LeftAntiJoin:
+                result = (sa.select([result])
+                          .where(~sa.exists(sa.select([1])
+                                            .where(onclause))))
             else:
                 raise NotImplementedError(jtype)
 
