@@ -182,7 +182,7 @@ class SelectBuilder(object):
 
         unchanged = True
 
-        if isinstance(op, ops.ValueNode):
+        if isinstance(op, ops.ValueOp):
             new_args = []
             for arg in op.args:
                 if isinstance(arg, ir.Expr):
@@ -276,7 +276,7 @@ class SelectBuilder(object):
         elif isinstance(op, (ops.Any, ops.BooleanValueOp,
                              ops.TableColumn, ir.Literal)):
             return expr
-        elif isinstance(op, ops.ValueNode):
+        elif isinstance(op, ops.ValueOp):
             visited = [self._visit_filter(arg)
                        if isinstance(arg, ir.Expr) else arg
                        for arg in op.args]
@@ -404,10 +404,8 @@ class SelectBuilder(object):
         self._collect(op.table, toplevel=toplevel)
 
     def _collect_Union(self, expr, toplevel=False):
-        if not toplevel:
-            return
-        else:
-            raise NotImplementedError
+        if toplevel:
+            raise NotImplementedError()
 
     def _collect_Aggregation(self, expr, toplevel=False):
         # The select set includes the grouping keys (if any), and these are
@@ -644,7 +642,7 @@ class _ExtractSubqueries(object):
             self._visit_join(expr)
         elif isinstance(node, ops.PhysicalTable):
             self._visit_physical_table(expr)
-        elif isinstance(node, ops.ValueNode):
+        elif isinstance(node, ops.ValueOp):
             for arg in node.flat_args():
                 if not isinstance(arg, ir.Expr):
                     continue
@@ -680,7 +678,9 @@ class _ExtractSubqueries(object):
         self.visit(expr.op().table)
 
     def _visit_Union(self, expr):
-        self.observe(expr)
+        op = expr.op()
+        self.visit(op.left)
+        self.visit(op.right)
 
     def _visit_MaterializedJoin(self, expr):
         self.observe(expr)
@@ -728,16 +728,28 @@ class _CorrelatedRefCheck(object):
         self._visit(self.expr)
         return self.has_query_root and self.has_foreign_root
 
-    def _visit(self, expr, in_subquery=False):
+    def _visit(self, expr, in_subquery=False, visit_cache=None,
+               visit_table_cache=None):
+        if visit_cache is None:
+            visit_cache = set()
+
+        if (id(expr), in_subquery) in visit_cache:
+            return
+        visit_cache.add((id(expr), in_subquery))
+
         node = expr.op()
 
         in_subquery = in_subquery or self._is_subquery(node)
 
         for arg in node.flat_args():
             if isinstance(arg, ir.TableExpr):
-                self._visit_table(arg, in_subquery=in_subquery)
+                self._visit_table(arg, in_subquery=in_subquery,
+                                  visit_cache=visit_cache,
+                                  visit_table_cache=visit_table_cache)
             elif isinstance(arg, ir.Expr):
-                self._visit(arg, in_subquery=in_subquery)
+                self._visit(arg, in_subquery=in_subquery,
+                            visit_cache=visit_cache,
+                            visit_table_cache=visit_table_cache)
             else:
                 continue
 
@@ -753,7 +765,15 @@ class _CorrelatedRefCheck(object):
 
         return False
 
-    def _visit_table(self, expr, in_subquery=False):
+    def _visit_table(self, expr, in_subquery=False, visit_cache=None,
+                     visit_table_cache=None):
+        if visit_table_cache is None:
+            visit_table_cache = set()
+
+        if (id(expr), in_subquery) in visit_table_cache:
+            return
+        visit_table_cache.add((id(expr), in_subquery))
+
         node = expr.op()
 
         if isinstance(node, (ops.PhysicalTable, ops.SelfReference)):
@@ -761,7 +781,9 @@ class _CorrelatedRefCheck(object):
 
         for arg in node.flat_args():
             if isinstance(arg, ir.Expr):
-                self._visit(arg, in_subquery=in_subquery)
+                self._visit(arg, in_subquery=in_subquery,
+                            visit_cache=visit_cache,
+                            visit_table_cache=visit_table_cache)
 
     def _ref_check(self, node, in_subquery=False):
         is_aliased = self.ctx.has_ref(node)
@@ -839,7 +861,7 @@ def _adapt_expr(expr):
         else:
             raise NotImplementedError(expr._repr())
 
-    elif isinstance(expr, ir.ArrayExpr):
+    elif isinstance(expr, ir.ColumnExpr):
         op = expr.op()
 
         def _get_column(name):
@@ -854,7 +876,7 @@ def _adapt_expr(expr):
             # Something more complicated.
             base_table = L.find_source_table(expr)
 
-            if isinstance(op, ops.DistinctArray):
+            if isinstance(op, ops.DistinctColumn):
                 expr = op.arg
                 try:
                     name = op.arg.get_name()
@@ -904,7 +926,7 @@ class QueryBuilder(object):
 
     def _make_union(self):
         op = self.expr.op()
-        return self._union_class(op.left, op.right,
+        return self._union_class(op.left, op.right, self.expr,
                                  distinct=op.distinct,
                                  context=self.context)
 
@@ -933,8 +955,8 @@ class QueryContext(object):
         self._table_key_memo = {}
         self.memo = memo or format.FormatMemo()
 
-    def _compile_subquery(self, expr, isolated=False):
-        sub_ctx = self.subcontext(isolated=isolated)
+    def _compile_subquery(self, expr):
+        sub_ctx = self.subcontext()
         return self._to_sql(expr, sub_ctx)
 
     def _to_sql(self, expr, ctx):
@@ -950,7 +972,7 @@ class QueryContext(object):
     def set_always_alias(self):
         self.always_alias = True
 
-    def get_compiled_expr(self, expr, isolated=False):
+    def get_compiled_expr(self, expr):
         this = self.top_context
 
         key = self._get_table_key(expr)
@@ -961,7 +983,7 @@ class QueryContext(object):
         if isinstance(op, ops.SQLQueryResult):
             result = op.query
         else:
-            result = self._compile_subquery(expr, isolated=isolated)
+            result = self._compile_subquery(expr)
 
         this.subquery_memo[key] = result
         return result
@@ -1016,11 +1038,8 @@ class QueryContext(object):
         self.extracted_subexprs.add(key)
         self.make_alias(expr)
 
-    def subcontext(self, isolated=False):
-        if not isolated:
-            return type(self)(indent=self.indent, parent=self)
-        else:
-            return type(self)(indent=self.indent)
+    def subcontext(self):
+        return type(self)(indent=self.indent, parent=self)
 
     # Maybe temporary hacks for correlated / uncorrelated subqueries
 
@@ -1418,10 +1437,11 @@ class TableSetFormatter(object):
 
 class Union(DDL):
 
-    def __init__(self, left_table, right_table, distinct=False,
+    def __init__(self, left_table, right_table, expr, distinct=False,
                  context=None):
         self.context = context
         self.left = left_table
         self.right = right_table
-
         self.distinct = distinct
+        self.table_set = expr
+        self.filters = []
