@@ -17,18 +17,20 @@ import locale
 import string
 import platform
 import warnings
+import operator
 
-from functools import reduce, partial
-from operator import add
+from functools import reduce
 
 import sqlalchemy as sa
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.sql.functions import GenericFunction
 
 from ibis.sql.alchemy import unary, varargs, fixed_arity, Over
+import ibis.expr.analytics as L
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
+import ibis.expr.window as W
 
 import ibis.sql.alchemy as alch
 _operation_registry = alch._operation_registry.copy()
@@ -283,7 +285,8 @@ def _strftime(t, expr):
 
 
 def _distinct_from(a, b):
-    return ((a != None) | (b != None)) & sa.func.coalesce(a != b, True)
+    return (((a != None) | (b != None)) &  # noqa
+            sa.func.coalesce(a != b, True))
 
 
 def _find_in_set(t, expr):
@@ -358,7 +361,7 @@ _cumulative_to_reduction = {
 
 
 def _cumulative_to_window(translator, expr, window):
-    win = ibis.cumulative_window()
+    win = W.cumulative_window()
     win = win.group_by(window._group_by).order_by(window._order_by)
 
     op = expr.op()
@@ -386,13 +389,15 @@ def _window(t, expr):
         ops.Lead,
         ops.DenseRank,
         ops.MinRank,
+        ops.NTile,
+        ops.PercentRank,
         ops.FirstValue,
         ops.LastValue,
     )
 
     if isinstance(window_op, ops.CumulativeOp):
-        arg = _cumulative_to_window(translator, arg, window)
-        return translator.translate(arg)
+        arg = _cumulative_to_window(t, arg, window)
+        return t.translate(arg)
 
     # Some analytic functions need to have the expression of interest in
     # the ORDER BY part of the window clause
@@ -405,18 +410,27 @@ def _window(t, expr):
 
     result = Over(
         reduction,
-        partition_by=partition_by or None,
-        order_by=order_by or None,
+        partition_by=partition_by,
+        order_by=order_by,
         preceding=window.preceding,
         following=window.following,
     )
 
-    if isinstance(window_op, (ops.RowNumber, ops.DenseRank, ops.MinRank)):
+    if isinstance(
+        window_op, (ops.RowNumber, ops.DenseRank, ops.MinRank, ops.NTile)
+    ):
         return result - 1
     else:
         return result
 
 
+def _ntile(t, expr):
+    op = expr.op()
+    args = op.args
+    arg, buckets = map(t.translate, args)
+    return sa.func.ntile(buckets)
+
+
 class regex_extract(GenericFunction):
     def __init__(self, string, pattern, index):
         super(regex_extract, self).__init__(string, pattern, index)
@@ -447,6 +461,56 @@ def _regex_extract(t, expr):
     )
 
 
+def _array_repeat(t, expr):
+    """Is this really that useful?
+
+    Repeat an array like a Python list using modular arithmetic,
+    scalar subqueries, and PostgreSQL's ARRAY function.
+
+    This is inefficient if PostgreSQL allocates memory for the entire sequence
+    and the output column. A quick glance at PostgreSQL's C code shows the
+    sequence is evaluated stepwise, which suggests that it's roughly constant
+    memory for the sequence generation.
+    """
+    raw, times = map(t.translate, expr.op().args)
+
+    # SQLAlchemy uses our column's table in the FROM clause. We need a simpler
+    # expression to workaround this.
+    array = sa.column(raw.name, type_=raw.type)
+
+    # We still need to prefix the table name to the column name in the final
+    # query, so make sure the column knows its origin
+    array.table = raw.table
+
+    array_length = sa.func.cardinality(array)
+
+    # sequence from 1 to the total number of elements desired in steps of 1.
+    # the call to greatest isn't necessary, but it provides clearer intent
+    # rather than depending on the implicit postgres generate_series behavior
+    start = step = 1
+    stop = sa.func.greatest(times, 0) * array_length
+    series = sa.func.generate_series(start, stop, step).alias()
+    series_column = sa.column(series.name, type_=sa.INTEGER)
+
+    # if our current index modulo the array's length
+    # is a multiple of the array's length, then the index is the array's length
+    index_expression = series_column % array_length
+    index = sa.func.coalesce(sa.func.nullif(index_expression, 0), array_length)
+
+    # tie it all together in a scalar subquery and collapse that into an ARRAY
+    selected = sa.select([array[index]]).select_from(series)
+    return sa.func.array(selected.as_scalar())
+
+
+def _identical_to(t, expr):
+    left, right = args = expr.op().args
+    if left.equals(right):
+        return True
+    else:
+        left, right = map(t.translate, args)
+        return left.op('IS NOT DISTINCT FROM')(right)
+
+
 _operation_registry.update({
     # types
     ops.Cast: _cast,
@@ -518,6 +582,21 @@ _operation_registry.update({
     ops.WindowOp: _window,
     ops.CumulativeOp: _window,
     ops.RowNumber: fixed_arity(lambda: sa.func.row_number(), 0),
+    ops.DenseRank: fixed_arity(lambda arg: sa.func.dense_rank(), 1),
+    ops.MinRank: fixed_arity(lambda arg: sa.func.rank(), 1),
+    ops.PercentRank: fixed_arity(lambda arg: sa.func.percent_rank(), 1),
+    ops.NTile: _ntile,
+
+    # array operations
+    ops.ArrayLength: fixed_arity(sa.func.cardinality, 1),
+    ops.ArrayCollect: fixed_arity(sa.func.array_agg, 1),
+    ops.ArraySlice: fixed_arity(
+        lambda array, start, stop: array[start:stop], 3
+    ),
+    ops.ArrayIndex: fixed_arity(operator.getitem, 2),
+    ops.ArrayConcat: fixed_arity(operator.add, 2),
+    ops.ArrayRepeat: _array_repeat,
+    ops.IdenticalTo: _identical_to,
 })
 
 
