@@ -12,7 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# flake8: noqa=E402
+
 import math
+import os
+import uuid
+import operator
 
 import pytest  # noqa
 
@@ -20,14 +25,17 @@ from .common import PostgreSQLTests
 from ibis.compat import unittest
 from ibis import literal as L
 import ibis.expr.types as ir
+import ibis.expr.datatypes as dt
 import ibis
 
-import sqlalchemy as sa
+sa = pytest.importorskip('sqlalchemy')
+pytest.importorskip('psycopg2')
 
 import pandas as pd
 import pandas.util.testing as tm
 
 
+@pytest.mark.postgresql
 class TestPostgreSQLFunctions(PostgreSQLTests, unittest.TestCase):
 
     def test_cast(self):
@@ -75,8 +83,8 @@ class TestPostgreSQLFunctions(PostgreSQLTests, unittest.TestCase):
         ic_casted = ic.cast('timestamp')
 
         # Logically, it's a timestamp
-        assert isinstance(tc_casted, ir.TimestampArray)
-        assert isinstance(ic_casted, ir.TimestampArray)
+        assert isinstance(tc_casted, ir.TimestampColumn)
+        assert isinstance(ic_casted, ir.TimestampColumn)
 
         cases = [
             (tc_casted, at.c.timestamp_col),
@@ -410,10 +418,6 @@ class TestPostgreSQLFunctions(PostgreSQLTests, unittest.TestCase):
         ]
         self._execute_projection(t, exprs)
 
-    @pytest.mark.xfail(
-        raises=sa.exc.ProgrammingError,
-        reason='union not working yet'
-    )
     def test_union(self):
         t = self.alltypes
 
@@ -654,3 +658,337 @@ class TestPostgreSQLFunctions(PostgreSQLTests, unittest.TestCase):
         expected = df.assign(new_col=[x / 2. for x in range(len(df))])
         result = expr['timestamp_col', 'new_col'].execute()
         tm.assert_frame_equal(result, expected)
+
+    def test_anonymous_aggregate(self):
+        t = self.alltypes
+        expr = t[t.double_col > t.double_col.mean()]
+        result = expr.execute()
+        df = t.execute()
+        expected = df[df.double_col > df.double_col.mean()].reset_index(
+            drop=True
+        )
+        tm.assert_frame_equal(result, expected)
+
+
+@pytest.fixture
+def con():
+    return ibis.postgres.connect(
+        host='localhost',
+        database=os.environ.get('IBIS_TEST_POSTGRES_DB', 'ibis_testing'),
+    )
+
+
+@pytest.fixture
+def array_types(con):
+    return con.table('array_types')
+
+
+@pytest.mark.postgresql
+def test_array_length(array_types):
+    expr = array_types.projection([
+        array_types.x.length().name('x_length'),
+        array_types.y.length().name('y_length'),
+        array_types.z.length().name('z_length'),
+    ])
+    result = expr.execute()
+    expected = pd.DataFrame({
+        'x_length': [3, 2, 2, 3, 3, 4],
+        'y_length': [3, 2, 2, 3, 3, 4],
+        'z_length': [3, 2, 2, 0, None, 4],
+    })
+
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+def test_array_schema(array_types):
+    assert array_types.x.type() == dt.Array(dt.int64)
+    assert array_types.y.type() == dt.Array(dt.string)
+    assert array_types.z.type() == dt.Array(dt.double)
+
+
+@pytest.mark.postgresql
+def test_array_collect(array_types):
+    expr = array_types.group_by(
+        array_types.grouper
+    ).aggregate(collected=array_types.scalar_column.collect())
+    result = expr.execute().sort_values('grouper').reset_index(drop=True)
+    expected = pd.DataFrame({
+        'grouper': list('abc'),
+        'collected': [[1.0, 2.0, 3.0], [4.0, 5.0], [6.0]],
+    })[['grouper', 'collected']]
+    tm.assert_frame_equal(result, expected, check_column_type=False)
+
+
+@pytest.mark.postgresql
+@pytest.mark.parametrize(
+    ['start', 'stop'],
+    [
+        (1, 3),
+        (1, 1),
+        (2, 3),
+        (2, 5),
+
+        (None, 3),
+        (None, None),
+        (3, None),
+
+        # negative slices are not supported
+        pytest.mark.xfail(
+            (-3, None),
+            raises=ValueError,
+            reason='Negative slicing not supported'
+        ),
+        pytest.mark.xfail(
+            (None, -3),
+            raises=ValueError,
+            reason='Negative slicing not supported'
+        ),
+        pytest.mark.xfail(
+            (-3, -1),
+            raises=ValueError,
+            reason='Negative slicing not supported'
+        ),
+    ]
+)
+def test_array_slice(array_types, start, stop):
+    expr = array_types[array_types.y[start:stop].name('sliced')]
+    result = expr.execute()
+    expected = pd.DataFrame({
+        'sliced': array_types.y.execute().map(lambda x: x[start:stop])
+    })
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+@pytest.mark.parametrize('index', [1, 3, 4, 11])
+def test_array_index(array_types, index):
+    expr = array_types[array_types.y[index].name('indexed')]
+    result = expr.execute()
+    expected = pd.DataFrame({
+        'indexed': array_types.y.execute().map(
+            lambda x: x[index] if index < len(x) else None
+        )
+    })
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+@pytest.mark.parametrize('n', [1, 3, 4, 7, -2])
+@pytest.mark.parametrize('mul', [lambda x, n: x * n, lambda x, n: n * x])
+def test_array_repeat(array_types, n, mul):
+    expr = array_types.projection([mul(array_types.x, n).name('repeated')])
+    result = expr.execute()
+    expected = pd.DataFrame({'repeated': array_types.x.execute().map(lambda x, n=n: mul(x, n))})
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+@pytest.mark.parametrize('catop', [lambda x, y: x + y, lambda x, y: y + x])
+def test_array_concat(array_types, catop):
+    t = array_types
+    x, y = t.x.cast('array<string>').name('x'), t.y
+    expr = t.projection([catop(x, y).name('catted')])
+    result = expr.execute()
+    tuples = t.projection([x, y]).execute().itertuples(index=False)
+    expected = pd.DataFrame({'catted': [catop(x, y) for x, y in tuples]})
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+def test_array_concat_mixed_types(array_types):
+    with pytest.raises(TypeError):
+        array_types.x + array_types.x.cast('array<double>')
+
+
+@pytest.yield_fixture
+def t(con):
+    name = 'left_t'
+    con.raw_sql(
+        """
+        CREATE TABLE {} (
+          id SERIAL PRIMARY KEY,
+          name TEXT
+        )
+        """.format(name)
+    )
+    try:
+        yield con.table(name)
+    finally:
+        con.drop_table(name)
+
+
+@pytest.yield_fixture
+def s(con, t):
+    name = 'right_t'
+    con.raw_sql(
+        """
+        CREATE TABLE {} (
+          id SERIAL PRIMARY KEY,
+          left_t_id INTEGER REFERENCES left_t,
+          cost DOUBLE PRECISION
+        )
+        """.format(name)
+    )
+    try:
+        yield con.table(name)
+    finally:
+        con.drop_table(name)
+
+
+@pytest.yield_fixture
+def trunc(con):
+    name = str(uuid.uuid1())
+    con.raw_sql(
+        """
+        CREATE TABLE "{}" (
+          id SERIAL PRIMARY KEY,
+          name TEXT
+        )
+        """.format(name)
+    )
+    con.raw_sql(
+        """INSERT INTO "{}" (name) VALUES ('a'), ('b'), ('c')""".format(name)
+    )
+    try:
+        yield con.table(name)
+    finally:
+        con.drop_table(name)
+
+
+@pytest.mark.postgresql
+def test_semi_join(t, s):
+    t_a, s_a = t.op().sqla_table.alias('t0'), s.op().sqla_table.alias('t1')
+    expr = t.semi_join(s, t.id == s.id)
+    result = expr.compile().compile(compile_kwargs=dict(literal_binds=True))
+    base = sa.select([t_a.c.id, t_a.c.name]).where(
+        sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id))
+    )
+    expected = sa.select([base.c.id, base.c.name])
+    assert str(result) == str(expected)
+
+
+@pytest.mark.postgresql
+def test_anti_join(t, s):
+    t_a, s_a = t.op().sqla_table.alias('t0'), s.op().sqla_table.alias('t1')
+    expr = t.anti_join(s, t.id == s.id)
+    result = expr.compile().compile(compile_kwargs=dict(literal_binds=True))
+    expected = sa.select([sa.column('id'), sa.column('name')]).select_from(
+        sa.select([t_a.c.id, t_a.c.name]).where(
+            ~sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id))
+        )
+    )
+    assert str(result) == str(expected)
+
+
+@pytest.mark.postgresql
+def test_create_table(con, trunc):
+    name = str(uuid.uuid1())
+    con.create_table(name, expr=trunc)
+    t = con.table(name)
+    assert list(t.name.execute()) == list('abc')
+
+
+@pytest.mark.postgresql
+def test_truncate_table(con, trunc):
+    assert list(trunc.name.execute()) == list('abc')
+    con.truncate_table(trunc.op().name)
+    assert not len(trunc.execute())
+
+
+@pytest.mark.postgresql
+def test_head(con):
+    t = con.table('functional_alltypes')
+    result = t.head().execute()
+    expected = t.limit(5).execute()
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.postgresql
+def test_identical_to(con):
+    # TODO: abstract this testing logic out into parameterized fixtures
+    t = con.table('functional_alltypes')
+    dt = t[['tinyint_col', 'double_col']].execute()
+    expr = t.tinyint_col.identical_to(t.double_col)
+    result = expr.execute()
+    expected = (dt.tinyint_col.isnull() & dt.double_col.isnull()) | (
+        dt.tinyint_col == dt.double_col
+    )
+    expected.name = result.name
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.postgresql
+def test_rank(con):
+    t = con.table('functional_alltypes')
+    expr = t.double_col.rank()
+    sqla_expr = expr.compile()
+    result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
+    expected = """\
+SELECT rank() OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp 
+FROM functional_alltypes AS t0"""
+    assert result == expected
+
+
+@pytest.mark.postgresql
+def test_percent_rank(con):
+    t = con.table('functional_alltypes')
+    expr = t.double_col.percent_rank()
+    sqla_expr = expr.compile()
+    result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
+    expected = """\
+SELECT percent_rank() OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS tmp 
+FROM functional_alltypes AS t0"""
+    assert result == expected
+
+
+@pytest.mark.postgresql
+def test_ntile(con):
+    t = con.table('functional_alltypes')
+    expr = t.double_col.ntile(7)
+    sqla_expr = expr.compile()
+    result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
+    expected = """\
+SELECT ntile(7) OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp 
+FROM functional_alltypes AS t0"""
+    assert result == expected
+
+
+@pytest.mark.parametrize('op', [operator.invert, operator.neg])
+def test_not_and_negate_bool(con, op):
+    t = con.table('functional_alltypes').limit(10)
+    expr = t.projection([op(t.bool_col).name('bool_col')])
+    result = expr.execute().bool_col
+    expected = op(t.execute().bool_col)
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.postgresql
+@pytest.mark.parametrize(
+    'field',
+    [
+        'tinyint_col',
+        'smallint_col',
+        'int_col',
+        'bigint_col',
+        'float_col',
+        'double_col',
+        'year',
+        'month',
+    ]
+)
+def test_negate_non_boolean(con, field):
+    t = con.table('functional_alltypes').limit(10)
+    expr = t.projection([(-t[field]).name(field)])
+    result = expr.execute()[field]
+    expected = -t.execute()[field]
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.postgresql
+def test_negate_boolean(con):
+    t = con.table('functional_alltypes').limit(10)
+    expr = t.projection([(-t.bool_col).name('bool_col')])
+    result = expr.execute().bool_col
+    expected = -t.execute().bool_col
+    tm.assert_series_equal(result, expected)
