@@ -12,8 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# flake8: noqa=E402
+
 import operator
 
+import pytest
+
+sa = pytest.importorskip('sqlalchemy')
+
 from ibis.compat import unittest
 from ibis.expr.tests.mocks import MockConnection
 from ibis.sql.tests.test_compiler import ExprTestCases
@@ -23,9 +29,10 @@ import ibis.expr.types as ir
 import ibis.sql.alchemy as alch
 import ibis
 
+
 from sqlalchemy import types as sat, func as F
 import sqlalchemy.sql as sql
-import sqlalchemy as sa
+
 
 L = sa.literal
 
@@ -121,8 +128,9 @@ class TestSQLAlchemySelect(unittest.TestCase, ExprTestCases):
 
         assert_equal(schema, expected)
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_ibis_to_sqla_conversion(self):
-        pass
+        assert False
 
     def test_comparisons(self):
         sat = self.sa_alltypes
@@ -507,11 +515,13 @@ class TestSQLAlchemySelect(unittest.TestCase, ExprTestCases):
 
         self._compare_sqla(expr, expected)
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_general_sql_function(self):
-        pass
+        assert False
 
+    @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_union(self):
-        pass
+        assert False
 
     def test_table_distinct(self):
         t = self.alltypes
